<?php
/**
* Class lsd config.
*
* @link       https://designersuiteforwp.com
* @since      1.0.0
*
* @package    Lsd
* @subpackage Lsd/public
*/

/**
* The
*
* @package    Lsd
* @subpackage Lsd/public
* @author     ORION <support@designersuiteforwp.com>
*/
class LSD_Config {

	/**
	* Register le custom post type
	*/
	public function lsd_register_cpt_config() {

		$labels = array(
			'name'               => _x( 'Configurations', 'lsd' ),
			'singular_name'      => _x( 'Configurations', 'lsd' ),
			'add_new'            => _x( 'New configuration', 'lsd' ),
			'add_new_item'       => _x( 'New configuration', 'lsd' ),
			'edit_item'          => _x( 'Edit configuration', 'lsd' ),
			'new_item'           => _x( 'New configuration', 'lsd' ),
			'view_item'          => _x( 'View configuration', 'lsd' ),
			'not_found'          => _x( 'No configuration found', 'lsd' ),
			'not_found_in_trash' => _x( 'No configuration in the trash', 'lsd' ),
			'menu_name'          => _x( 'Product Designer', 'lsd' ),
			'all_items'          => _x( 'Configurations', 'lsd' ),
		);

		$args = array(
			'labels'              => $labels,
			'hierarchical'        => false,
			'description'         => 'Configurations',
			'supports'            => array( 'title' ),
			'public'              => false,
			'show_ui'             => true,
			'show_in_menu'        => false,
			'show_in_nav_menus'   => false,
			'publicly_queryable'  => false,
			'exclude_from_search' => true,
			'has_archive'         => false,
			'query_var'           => false,
			'can_export'          => true,
		);

		register_post_type( 'lsd-config', $args );
	}

	/**
	* Builds the cpt metaboxes
	*/
	public function lsd_get_config_metabox() {

		$screens = array( 'lsd-config' );

		foreach ( $screens as $screen ) {

			add_meta_box(
				'lsd-metas-canvas', __( 'Canvas', 'lsd' ), array( $this, 'lsd_get_config_canvas_metabox' ), $screen
			);

			add_meta_box(
				'lsd-metas-text', __( 'Text', 'lsd' ), array( $this, 'lsd_get_config_text_metabox' ), $screen
			);

			add_meta_box(
				'lsd-metas-fonts', __( 'Fonts', 'lsd' ), array( $this, 'lsd_get_config_fonts_metabox' ), $screen
			);

			add_meta_box(
				'lsd-metas-pricing-rules', __( 'Pricing rules', 'lsd' ), array( $this, 'lsd_get_config_pricing_metabox' ), $screen
			);
		}
	}

	/**
	* Builds the canvas meta box
	*/
	public function lsd_get_config_canvas_metabox() {
		wp_enqueue_media();
		wp_nonce_field( basename( __FILE__ ), 'lsd_config_nonce' );
		$args      = array( 'post_status' => 'publish' );
		$pages     = get_pages( $args );
		$pages_arr = array();

		foreach ( $pages as $page ) {
			$pages_arr[ $page->ID ] = $page->post_title;
		}
		$begin      = array(
			'type' => 'sectionbegin',
			'id'   => 'lsd-canvas-settings',
		);
		$max_width  = array(
			'title'   => __( 'Default width', 'lsd' ),
			'name'    => 'lsd-metas[canvas-w]',
			'type'    => 'number',
			'desc'    => __( 'in pixels. Canvas max width. Leave this field empty to use the value defined in the global settings.', 'lsd' ),
			'default' => 144,
		);
		$max_height = array(
			'title'   => __( ' Default height', 'lsd' ),
			'name'    => 'lsd-metas[canvas-h]',
			'type'    => 'number',
			'desc'    => __( 'in pixels. Canvas max width. Leave this field empty to use the value defined in the global settings.', 'lsd' ),
			'default' => 30,
		);

		$dimensions_unit = array(
			'title'   => __( 'Unit', 'lsd' ),
			'name'    => 'lsd-metas[canvas-dimensions-unit]',
			'type'    => 'select',
			'options' => array(
				'pt' => __( 'Point', 'lsd' ),
				'mm' => __( 'Millimeter', 'lsd' ),
				'cm' => __( 'Centimeter', 'lsd' ),
				'px' => __( 'Pixels', 'lsd' ),
				'in' => __( 'Inch', 'lsd' ),
			),
			'default' => 'mm',
		);

		$dimensions = array(
			'title'  => __( 'Default product dimensions', 'lsd' ),
			'type'   => 'groupedfields',
			'desc'   => __( 'Width and Height of the design area in the browser. <span style="color: red">It can be smaller than the expected output files in order to fit inside the website pages but need to be a scaled version of the expected output.</span>', 'lsd' ),
			'fields' => array( $max_width, $max_height, $dimensions_unit ),
		);

		$keep_ratio = array(
			'title'   => __( 'Keep ratio', 'lsd' ),
			'name'    => 'lsd-metas[keep-ratio]',
			'default' => 'no',
			'desc'    => __( 'Wether or not the customer can ajust his design dimensions by keeping the ratio.', 'lsd' ),
			'type'    => 'radio',
			'options' => array(
				'no'  => __( 'No', 'lsd' ),
				'yes' => __( 'Yes', 'lsd' ),
			),
		);

		$bg_type = array(
			'title'     => __( 'Background color selector', 'lsd' ),
			'name'      => 'lsd-metas[bg-type]',
			'default'   => 'colors',
			'type'      => 'radio',
			'desc'      => __( 'How the customer will set the canvas background color.', 'lsd' ),
			'options'   => array(
				'colors'      => __( 'Colors Palette', 'lsd' ),
				'colorpicker' => __( 'Colors Picker', 'lsd' ),
				'none'        => __( 'None', 'lsd' ),
			),
			'row_class' => 'lsd-bg-type',
		);

		$price = array(
			'title' => __( 'Price', 'lsd' ),
			'name'  => 'price',
			'type'  => 'number',
		);

		$hex_code = array(
			'title' => __( 'HEX color code', 'lsd' ),
			'name'  => 'hex',
			'type'  => 'color',
		);

		$default_hex_code = array(
			'title'     => __( 'Default color', 'lsd' ),
			'name'      => 'lsd-metas[default-hex-code]',
			'type'      => 'color',
			'row_class' => 'show-if-canvas-colorpicker',
		);

		$bg_colors = array(
			'title'         => __( 'Palette', 'lsd' ),
			'name'          => 'lsd-metas[bg-colors-palette]',
			'type'          => 'repeatable-fields',
			'fields'        => array( $hex_code, $price ),
			'desc'          => __( 'Text color palette', 'lsd' ),
			'class'         => 'striped',
			'add_btn_label' => __( 'Add color', 'lsd' ),
			'row_class'     => 'show-if-canvas-colors',
		);

		$end      = array( 'type' => 'sectionend' );
		$settings = array(
			$begin,
			$dimensions,
			$keep_ratio,
			$bg_type,
			$bg_colors,
			$default_hex_code,
			$end,
		);
		//Escaping output.
		$allowed_tags = lsd_allowed_tags();
		$raw_output = o_admin_fields( $settings );
		echo wp_kses( $raw_output, $allowed_tags);
		global $o_row_templates;
		?>
		<script>
		var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
		</script>
		<?php
	}

	/**
	* Builds the text meta box
	*/
	public function lsd_get_config_text_metabox() {
		wp_enqueue_media();

		$args      = array( 'post_status' => 'publish' );
		$pages     = get_pages( $args );
		$pages_arr = array();

		foreach ( $pages as $page ) {
			$pages_arr[ $page->ID ] = $page->post_title;
		}
		$begin = array(
			'type' => 'sectionbegin',
			'id'   => 'lsd-text-settings',
		);

		$bg_type = array(
			'title'     => __( 'Color Type', 'lsd' ),
			'name'      => 'lsd-metas[text-color-type]',
			'default'   => 'colors',
			'type'      => 'radio',
			'desc'      => __( 'How the customer will set the text color.', 'lsd' ),
			'options'   => array(
				'colors'      => __( 'Colors Palette', 'lsd' ),
				'colorpicker' => __( 'Colors Picker', 'lsd' ),
			),
			'row_class' => 'lsd-text-color-type',
		);

		$price = array(
			'title' => __( 'Price', 'lsd' ),
			'name'  => 'price',
			'type'  => 'number',
		);

		$hex_code = array(
			'title' => __( 'HEX color code', 'lsd' ),
			'name'  => 'hex',
			'type'  => 'color',
		);

		$default_hex_code = array(
			'title'     => __( 'HEX color code', 'lsd' ),
			'name'      => 'lsd-metas[default-text-color]',
			'type'      => 'color',
			'row_class' => 'show-if-text-colorpicker',
		);

		$bg_colors = array(
			'title'         => __( 'Palette', 'lsd' ),
			'name'          => 'lsd-metas[text-colors-palette]',
			'type'          => 'repeatable-fields',
			'fields'        => array( $hex_code, $price ),
			'desc'          => __( 'Text color palette', 'lsd' ),
			'class'         => 'striped',
			'add_btn_label' => __( 'Add color', 'lsd' ),
			'row_class'     => 'show-if-text-colors',
		);

		$end      = array( 'type' => 'sectionend' );
		$settings = array(
			$begin,
			$bg_type,
			$bg_colors,
			$default_hex_code,
			$end,
		);
		//Escaping output.
		$allowed_tags = lsd_allowed_tags();
		$raw_output = o_admin_fields( $settings );
		echo wp_kses( $raw_output, $allowed_tags);
		global $o_row_templates;
		?>
		<script>
		var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
		</script>
		<?php
	}

	/**
	* Builds the fonts meta box
	*/
	public function lsd_get_config_fonts_metabox() {
		wp_enqueue_media();

		$args      = array( 'post_status' => 'publish' );
		$pages     = get_pages( $args );
		$pages_arr = array();

		foreach ( $pages as $page ) {
			$pages_arr[ $page->ID ] = $page->post_title;
		}
		$begin = array(
			'type' => 'sectionbegin',
			'id'   => 'lsd-fonts-settings',
		);

		$use_global_fonts = array(
			'title'     => __( 'Use global fonts', 'lsd' ),
			'name'      => 'lsd-metas[global-fonts]',
			'default'   => 'yes',
			'desc'      => __( 'Available fonts to use when designing.', 'lsd' ),
			'type'      => 'radio',
			'options'   => array(
				'no'  => __( 'No', 'lsd' ),
				'yes' => __( 'Yes', 'lsd' ),
			),
			'row_class' => 'lsd-fonts-type',
		);

		$lsd_fonts    = lsd_refactor_fonts( get_option( 'lsd-fonts' ) );
		$chosen_fonts = array(
			'title'             => __( 'Selected fonts', 'lsd' ),
			'name'              => 'lsd-metas[selected-fonts]',
			'desc'              => __( 'Please select the fonts that should be available for use.', 'lsd' ),
			'type'              => 'select',
			'options'           => $lsd_fonts,
			'row_class'         => 'show-if-fonts-global-no',
			'id'                => 'lsd_fonts_selector',
			'custom_attributes' => array(
				'multiple' => 'multiple',
			),
		);

		$end      = array( 'type' => 'sectionend' );
		$settings = array(
			$begin,
			$use_global_fonts,
			$chosen_fonts,
			$end,
		);
		//Escaping output.
		$allowed_tags = lsd_allowed_tags();
		$raw_output = o_admin_fields( $settings );
		echo wp_kses( $raw_output, $allowed_tags);
		global $o_row_templates;
		?>
		<script>
		var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
		</script>
		<?php
	}

	/**
	* Builds the pricing metabox
	*/
	public function lsd_get_config_pricing_metabox() {
		$post_id    = get_the_ID();
		$lsd_metas  = get_post_meta( $post_id, 'lsd-metas', true );
		$allowed_tags = lsd_allowed_tags();

		$params     = array(
			'if_canvas_square_unit' => __( 'If Canvas square unit (WxH)', 'lsd' ),
			'txt_nb_chars'          => __( 'NB chars in text', 'lsd' ),
			'txt_nb_lines'          => __( 'NB lines in text', 'lsd' ),
			'if_selected_font'      => __( 'If selected font', 'lsd' ),
			'if_char_spacing'       => __( 'If Character spacing', 'lsd' ),
		);
		$first_rule = $this->lsd_get_rule_tpl( $params, true );

		$rule_tpl   = $this->lsd_get_rule_tpl( $params, false );
		?>
		<div class='lsd-rules-table-container'>
			<textarea id='lsd-rule-tpl' style='display: none;'>
				<?php echo esc_textarea($rule_tpl ); ?>
			</textarea>
			<textarea id='lsd-first-rule-tpl' style='display: none;'>
				<?php echo esc_textarea($first_rule); ?>
			</textarea>
			<?php
			$pricing_rules = array();
			if ( isset( $lsd_metas['pricing-rules'] ) ) {
				$pricing_rules = $lsd_metas['pricing-rules'];
			}
			if ( is_array( $pricing_rules ) && ! empty( $pricing_rules ) ) {
				$rule_group = 0;
				foreach ( $pricing_rules as $rules_group ) {
					$rule_index = 0;
					$rules      = $rules_group['rules'];
					$a_price    = $rules_group['a_price'];
					if (isset($rules_group["scope"])) {
						$scope = $rules_group["scope"];
					}else {
						$scope = 'item';
					}
					?>
					<table class="lsd-rules-table widefat">
						<tbody>
							<?php
							foreach ( $rules as $rule_arr ) {
								if ( 0 === $rule_index ) {
									$rule_html = $this->lsd_get_rule_tpl( $params, true, $rule_arr['param'], $rule_arr['operator'], $rule_arr['value'], $a_price, $scope, count( $rules ) );
								} else {
									$rule_html = $this->lsd_get_rule_tpl( $params, false, $rule_arr['param'], $rule_arr['operator'], $rule_arr['value'] );
								}
								$r1 = str_replace( '{rule-group}', $rule_group, $rule_html );
								$r2 = str_replace( '{rule-index}', $rule_index, $r1 );
								//echo wp_kses($r2, $allowed_tags );
								echo $r2;
								$rule_index++;
							}
							$rule_group++;
							?>
						</tbody>
					</table>
					<?php
				}
			} else {
				?>
				<table class="lsd-rules-table widefat">
					<tbody>
						<?php
						$rule_group = 0;
						$rule_index = 0;
						$r1         = str_replace( '{rule-group}', $rule_group, $first_rule );
						$r2         = str_replace( '{rule-index}', $rule_index, $r1 );
						//Escaping output.
						echo wp_kses( $r2, $allowed_tags);
						?>
					</tbody>
				</table>
				<?php
			}
			?>

		</div>
		<a class="button lsd-add-group mg-bot-10i">Add rule group</a>
		<?php
	}

	/**
	* Builds the output metabox
	*/
	public function lsd_get_config_output_metabox() {
		global $lsd_settings;
		$output_global_settings = get_proper_value( $lsd_settings, 'lsd-output-options', array() );

		$options           = array();
		$output_format     = array(
			'title'     => __( 'Output Format', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][output-format]',
			'type'      => 'radio',
			'row_class' => 'config-output-format',
			'options'   => array(
				'png'     => __( 'PNG', 'lsd' ),
				'jpg'     => __( 'JPG', 'lsd' ),
				'svg'     => __( 'SVG', 'lsd' ),
				'pdf+png' => __( 'PDF + PNG', 'lsd' ),
				'pdf+jpg' => __( 'PDF + JPG', 'lsd' ),
				'pdf+svg' => __( 'PDF + SVG', 'lsd' ),
			),
			'default'   => 'png',
		);
		$output_w          = array(
			'title'     => __( 'Output width (px)', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][lsd-min-output-width]',
			'type'      => 'text',
			'row_class' => 'hide-if-pdf',
			'default'   => get_proper_value( $output_global_settings, 'lsd-min-output-width' ),
		);
		$output_loop_delay = array(
			'title'   => __( 'Output loop delay (milliseconds)', 'lsd' ),
			'name'    => 'lsd-metas[output-settings][lsd-output-loop-delay]',
			'type'    => 'text',
			'default' => get_proper_value( $output_global_settings, 'lsd-output-loop-delay' ),
		);

		$zip_output      = array(
			'title'     => __( 'Zip output file', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][zip-output]',
			'type'      => 'radio',
			'row_class' => 'zip-output',
			'options'   => array(
				'yes' => __( 'Yes', 'lsd' ),
				'no'  => __( 'No', 'lsd' ),
			),
			'default'   => 'no',
		);
		$zip_folder_name = array(
			'title'     => __( 'Zip output folder prefix', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][zip-folder-name]',
			'type'      => 'text',
			'row_class' => 'show-if-zip',
		);
		$cmyk_conversion = array(
			'title'     => __( 'CMYK conversion (Requires ImageMagick)', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][lsd-cmyk-conversion]',
			'type'      => 'radio',
			'row_class' => 'show-if-jpg',
			'options'   => array(
				'no'  => __( 'No', 'lsd' ),
				'yes' => __( 'Yes', 'lsd' ),
			),
			'default'   => get_proper_value( $output_global_settings, 'lsd-cmyk-conversion' ),
		);

		$pdf_format = array(
			'title'     => __( 'PDF Format', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][pdf-format]',
			'type'      => 'groupedselect',
			'desc'      => __( 'Output PDF format and dimensions', 'lsd' ),
			'data-id'   => '',
			'row_class' => 'show-if-pdf',
			'options'   => get_lsd_pdf_formats(),
			'default'   => get_proper_value( $output_global_settings, 'pdf-format' ),
		);

		$pdf_w = array(
			'title'   => __( 'Width', 'lsd' ),
			'name'    => 'lsd-metas[output-settings][pdf-width]',
			'type'    => 'number',
			'default' => get_proper_value( $output_global_settings, 'pdf-width' ),
		);

		$pdf_h = array(
			'title'   => __( 'Height', 'lsd' ),
			'name'    => 'lsd-metas[output-settings][pdf-height]',
			'type'    => 'number',
			'default' => get_proper_value( $output_global_settings, 'pdf-height' ),
		);

		$pdf_unit = array(
			'title'   => __( 'Unit', 'lsd' ),
			'name'    => 'lsd-metas[output-settings][pdf-unit]',
			'type'    => 'select',
			'class'   => 'pdf-unit',
			'options' => array(
				'pt' => __( 'Point', 'lsd' ),
				'mm' => __( 'Millimeter', 'lsd' ),
				'cm' => __( 'Centimeter', 'lsd' ),
				'px' => __( 'Pixels', 'lsd' ),
				'in' => __( 'Inch', 'lsd' ),
			),
			'default' => get_proper_value( $output_global_settings, 'pdf-unit' ),
		);

		$pdf_custom_dimensions = array(
			'title'     => 'Custom PDF dimensions',
			'desc'      => __( 'These dimensions will only be used if the PDF format is set to Custom.', 'lsd' ),
			'type'      => 'groupedfields',
			'row_class' => 'show-if-pdf',
			'fields'    => array( $pdf_w, $pdf_h, $pdf_unit ),
		);
		$pdf_resolution        = array(
			'title'     => 'PDF resolution (dpi)',
			'desc'      => __( 'Output PDF resolution.', 'lsd' ),
			'type'      => 'number',
			'name'      => 'lsd-metas[output-settings][resolution]',
			'row_class' => 'hide-if-pixels show-if-pdf',
			'default'   => 300,
		);

		$pdf_margin_top_bottom = array(
			'title'   => __( 'PDF Margin Top & Bottom', 'lsd' ),
			'name'    => 'lsd-metas[output-settings][pdf-margin-tb]',
			'type'    => 'text',
			'default' => get_proper_value( $output_global_settings, 'pdf-margin-tb', 20 ),
		);

		$pdf_margin_left_right = array(
			'title'   => __( 'PDF Margin Left & Right', 'lsd' ),
			'name'    => 'lsd-metas[output-settings][pdf-margin-lr]',
			'type'    => 'text',
			'default' => get_proper_value( $output_global_settings, 'pdf-margin-lr', 20 ),
		);

		$margins = array(
			'title'     => __( 'Margins', 'lsd' ),
			'type'      => 'groupedfields',
			'desc'      => __( 'Vertical and Horizontal margins in the PDF file.', 'lsd' ),
			'row_class' => 'show-if-pdf',
			'fields'    => array( $pdf_margin_top_bottom, $pdf_margin_left_right ),
		);

		$crop_x = array(
			'title' => __( 'Center Abcissa', 'lsd' ),
			'name'  => 'x',
			'type'  => 'number',
		);
		$crop_y = array(
			'title' => __( 'Center Ordinate', 'lsd' ),
			'name'  => 'y',
			'type'  => 'number',
		);
		$crop_w = array(
			'title' => __( 'Width', 'lsd' ),
			'name'  => 'w',
			'type'  => 'number',
		);
		$crop_h = array(
			'title' => __( 'Height', 'lsd' ),
			'name'  => 'h',
			'type'  => 'number',
		);
		$type   = array(
			'title' => __( 'Type', 'lsd' ),
			'name'  => 'type',
			'type'  => 'text',
		);

		$crop_marks = array(
			'title'           => __( 'Crop marks', 'lsd' ),
			'name'            => 'lsd-metas[output-settings][crop-marks]',
			'desc'            => __( 'Crop marks settings.<br> <strong>Type</strong>: One symbol per type separated by comma: <ul><li>T = TOP</li><li>F = BOTTOM</li> <li>L = LEFT</li> <li>R = RIGHT</li><li>TL = A = TOP-LEFT</li><li>TR = B = TOP-RIGHT</li><li>BL = C = BOTTOM-LEFT</li><li>BR = D = BOTTOM-RIGHT</li></ul>', 'lsd' ),
			'type'            => 'repeatable-fields',
			'fields'          => array( $crop_x, $crop_y, $crop_w, $crop_h, $type ),
			'row_class'       => 'hide-if-pixels show-if-pdf',
			'ignore_desc_col' => false,
			'add_btn_label'   => __( 'Add Mark', 'lsd' ),
		);

		$bleed_x = array(
			'title'             => __( 'H. spacing', 'lsd' ),
			'name'              => 'h',
			'type'              => 'number',
			'custom_attributes' => array( 'step' => '0.01' ),
		);
		$bleed_y = array(
			'title'             => __( 'V. spacing', 'lsd' ),
			'name'              => 'v',
			'type'              => 'number',
			'custom_attributes' => array( 'step' => '0.01' ),
		);

		$bleed_color = array(
			'title'             => __( 'Border color', 'lsd' ),
			'name'              => 'color',
			'type'              => 'text',
			'custom_attributes' => array( 'placeholder' => '#FF0000 for ex.' ),
		);

		$bleed_dash_spacing = array(
			'title'             => __( 'Dash spacing', 'lsd' ),
			'name'              => 'dash-spacing',
			'type'              => 'number',
			'custom_attributes' => array( 'step' => '0.01' ),
		);

		$bleed_border_width = array(
			'title'             => __( 'Border width', 'lsd' ),
			'name'              => 'border-width',
			'type'              => 'number',
			'custom_attributes' => array( 'step' => '0.01' ),
		);

		$bleed = array(
			'title'           => __( 'Bleed, Trim & Safe zones', 'lsd' ),
			'name'            => 'lsd-metas[output-settings][zones]',
			'desc'            => __( '<ul><li><strong>H. spacing</strong>: Space between the left and right borders of the document and the area.</li><li><strong>V. spacing</strong>: Space between the top and bottom borders of the document and the area.</li></ul>', 'lsd' ),
			'type'            => 'repeatable-fields',
			'fields'          => array( $bleed_x, $bleed_y, $bleed_color, $bleed_dash_spacing, $bleed_border_width ),
			'row_class'       => 'hide-if-pixels show-if-pdf',
			'ignore_desc_col' => false,
			'add_btn_label'   => __( 'Add zone', 'lsd' ),
		);

		$pdf_orientation = array(
			'title'     => __( 'PDF Orientation', 'lsd' ),
			'name'      => 'lsd-metas[output-settings][pdf-orientation]',
			'default'   => 'P',
			'type'      => 'select',
			'row_class' => 'show-if-pdf',
			'options'   => array(
				'P' => __( 'Portrait', 'lsd' ),
				'L' => __( 'Landscape', 'lsd' ),
			),
			'default'   => get_proper_value( $output_global_settings, 'pdf-orientation' ),
		);

		$output_options_begin = array(
			'type'  => 'sectionbegin',
			'title' => __( 'Output Settings', 'lsd' ),
			'table' => 'metas',
			'id'    => 'lsd_product_output',
		);

		$output_options_end = array(
			'type' => 'sectionend',
			'id'   => 'lsd_product_output',
		);

		array_push( $options, $output_options_begin );
		array_push( $options, $output_format );
		array_push( $options, $output_w );
		array_push( $options, $output_loop_delay );
		array_push( $options, $cmyk_conversion );
		array_push( $options, $pdf_format );
		array_push( $options, $pdf_custom_dimensions );
		array_push( $options, $margins );
		array_push( $options, $crop_marks );
		array_push( $options, $bleed );
		array_push( $options, $pdf_resolution );
		array_push( $options, $pdf_orientation );
		array_push( $options, $zip_output );
		array_push( $options, $zip_folder_name );
		array_push( $options, $output_options_end );
		?>

		<?php
		//Escaping output.
		$allowed_tags = lsd_allowed_tags();
		$raw_output = o_admin_fields( $settings );
		echo wp_kses( $raw_output, $allowed_tags);
		global $o_row_templates;
		?>
		<script>
		var o_rows_tpl = <?php echo wp_json_encode( $o_row_templates ); ?>;
		</script>
		<?php
	}

	/**
	* Builds the pricing rules table
	*
	* @param type $params Array of parameters.
	* @param type $with_price Use with price.
	* @param type $default_param Default parameter.
	* @param type $default_operator Default operator.
	* @param type $default_value Default value.
	* @param type $default_price Default price.
	* @param type $default_scope Default scope.
	* @param type $count Count.
	* @return type
	*/
	private function lsd_get_rule_tpl( $params, $with_price = false, $default_param = false, $default_operator = '<', $default_value = '', $default_price = '', $default_scope = "per_item", $count = 1 ) {
		ob_start();
		$operators = array(
			'<'  => __( 'is less than', 'lsd' ),
			'<=' => __( 'is less or equal to', 'lsd' ),
			'==' => __( 'equals', 'lsd' ),
			'>'  => __( 'more than', 'lsd' ),
			'>=' => __( 'more or equal to', 'lsd' ),
			'!=' => __( 'not equals', 'lsd' ),
		);
		$scopes = array(
			"item" => __("Per item", "wpd"),
			"design" => __("On whole design", "wpd")
		);
		$allowed_tags = lsd_allowed_tags();
		?>
		<tr data-id="rule_{rule-group}">
			<td class="param">
				<select id="lsd-group_{rule-group}_rule_{rule-index}_param" class="select lsd-pricing-group-param" name="lsd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][param]">
					<?php
					foreach ( $params as $param_key => $param_val ) {
						if ( $param_key === $default_param ) {
							?>
							<option value='<?php echo esc_attr( $param_key ); ?>' selected="selected"><?php echo esc_attr( $param_val ); ?></option>
							<?php
						} else {
							?>
							<option value='<?php echo esc_attr( $param_key ); ?>'><?php echo esc_attr( $param_val ); ?></option>
							<?php
						}
					}
					?>
				</select>
			</td>
			<td class="operator">
				<select id="lsd-pricing-group_{rule-group}_rule_{rule-index}_operator" class="select" name="lsd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][operator]">
					<?php
					$option = 0;
					foreach ( $operators as $operator_key => $operator_val ) {
						if ( $operator_key === $default_operator ) {
							$op_html = "<option id='lsd-pricing-group_{rule-group}_rule_{rule-index}_{option}' value='" . htmlentities($operator_key) . "' selected='selected'>" . $operator_val . '</option>';
						} else {
							$op_html = "<option id='lsd-pricing-group_{rule-group}_rule_{rule-index}_{option}' value='" . htmlentities($operator_key) . "'>" . $operator_val  . '</option>';
						}
						$op = str_replace( '{option}', $option, $op_html );
						echo wp_kses($op, $allowed_tags);
						$option++;
					}
					?>
					<script type="text/javascript">
					var operators=<?php echo wp_json_encode( $operators ); ?> ;
					</script>

				</select>
			</td>
			<td class="value">
				<input type="text" name="lsd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][value]" value="<?php echo esc_attr( $default_value ); ?>" placeholder="number">
			</td>
			<?php
			if ( $with_price ) {
				?>
				<td class="a_price" rowspan="<?php echo esc_attr( $count ); ?>">
					<input type="number" name="lsd-metas[pricing-rules][group_{rule-group}][a_price]" value="<?php echo esc_attr( $default_price ); ?>" placeholder="price" step="any">
					<select id="lsd-pricing-group_{rule-group}_rule_{rule-index}_scope" class="select" name="lsd-metas[pricing-rules][group_{rule-group}][scope]">
						<?php
						foreach ($scopes as $scope_key => $scope_val) {
							if ($scope_key == $default_scope) {
								?><option value='<?php echo $scope_key; ?>' selected="selected"><?php echo $scope_val; ?></option><?php
							} else {
								?><option value='<?php echo $scope_key; ?>'><?php echo $scope_val; ?></option><?php
							}
						}
						?>
					</select>
				</td>
				<?php
			}
			?>
			<td class="add">
				<a class="lsd-add-rule button" data-group='{rule-group}'><?php echo esc_attr( 'and', 'lsd' ); ?></a>
			</td>
			<td class="remove">
				<a class="lsd-remove-rule acf-button-remove"></a>
			</td>
		</tr>
		<?php
		$rule_tpl = ob_get_contents();
		ob_end_clean();
		return $rule_tpl;
	}

	/**
	* Save the config
	*
	* @param type $post_id Post id.
	*/
	public function lsd_save_config( $post_id ) {
		$meta_key   = 'lsd-metas';
		$checkboxes = array( 'can-design-from-blank' );
		if ( isset( $_POST[ $meta_key ]) && wp_verify_nonce( $_POST[ 'lsd_config_nonce' ], basename( __FILE__ ) ) ) {// Input var okay.
			$old_metas = get_post_meta( $post_id, $meta_key, true );
			if ( empty( $old_metas ) ) {
				$old_metas = array();
			}
			$new_metas = array_replace( $old_metas, $_POST[ $meta_key ] );// Input var okay.
			foreach ( $checkboxes as $checkboxes_key ) {
				if ( ! isset( $_POST[ $meta_key ][ $checkboxes_key ] ) && isset( $new_metas[ $checkboxes_key ] ) ) {// Input var okay.
					unset( $new_metas[ $checkboxes_key ] );
				}
			}
			update_post_meta( $post_id, $meta_key, $new_metas );
		}
	}

	/**
	* Re order the metaboxes
	*
	* @param array $order Array of ordered elements.
	* @return string
	*/
	public function lsd_get_metabox_order( $order ) {
		$order['advanced'] = 'lsd-metas-canvas,lsd-metas-text,lsd-metas-fonts,lsd-metas-pricing-rules,lsd-metas-output,submitdiv';
		return $order;
	}

	/**
	* Duplicate the config link
	*
	* @param string $actions Actions.
	* @param type   $post Post object.
	* @return string
	*/
	public function lsd_duplicate_post_link( $actions, $post ) {
		if ( 'lsd-config' === $post->post_type && current_user_can( 'edit_posts' ) ) {
			$actions['duplicate'] = '<a href="admin.php?action=lsd_duplicate_config&amp;post=' . $post->ID . '&amp;duplicate_nonce=' . wp_create_nonce( basename( __FILE__ ) ) . '" title="Duplicate this item" rel="permalink">Duplicate</a>';
		}
		return $actions;
	}

	/**
	* Duplicate the configs
	*/
	public function lsd_duplicate_config() {
		global $lsdb;

		/*
		* Nonce verification
		*/
		if ( ! isset( $_GET['duplicate_nonce'] ) || ! wp_verify_nonce( $_GET['duplicate_nonce'], basename( __FILE__ ) ) ) {
			return;
		}else{
			if ( ! ( isset( $_GET['post'] ) || isset( $_POST['post'] ) || ( isset( $_REQUEST['action'] ) && 'lsd_duplicate_config' === sanitize_text_field( wp_unslash( $_REQUEST['action'] ) ) ) ) ) {// Input is ok.
				wp_die( 'No post to duplicate has been supplied!' );
			}
		}

		/*
		* get the original post id
		*/
		$post_id = ( isset( $_GET['post'] ) ? absint( $_GET['post'] ) : absint( $_POST['post'] ) );

		/*
		* and all the original post data then
		*/
		$post = get_post( $post_id );

		/*
		* if you don't want current user to be the new post author,
		* then change next couple of lines to this: $new_post_author = $post->post_author;
		*/
		$current_user    = wp_get_current_user();
		$new_post_author = $current_user->ID;

		/*
		* if post data exists, create the post duplicate
		*/
		if ( isset( $post ) && null !== $post ) {
			/*
			* new post data array
			*/
			$args = array(
				'comment_status' => $post->comment_status,
				'ping_status'    => $post->ping_status,
				'post_author'    => $new_post_author,
				'post_content'   => $post->post_content,
				'post_excerpt'   => $post->post_excerpt,
				'post_name'      => $post->post_name,
				'post_parent'    => $post->post_parent,
				'post_password'  => $post->post_password,
				'post_status'    => 'draft',
				'post_title'     => $post->post_title . __( ' - copy', 'lsd' ),
				'post_type'      => $post->post_type,
				'to_ping'        => $post->to_ping,
				'menu_order'     => $post->menu_order,
			);

			/*
			* insert the post by wp_insert_post() function
			*/
			$new_post_id = wp_insert_post( $args );

			/*
			* get all current post terms ad set them to the new post draft
			*/
			$taxonomies = get_object_taxonomies( $post->post_type ); // returns array of taxonomy names for post type, ex array("category", "post_tag");.
			foreach ( $taxonomies as $taxonomy ) {
				$post_terms = wp_get_object_terms( $post_id, $taxonomy, array( 'fields' => 'slugs' ) );
				wp_set_object_terms( $new_post_id, $post_terms, $taxonomy, false );
			}

			/*
			* duplicate all post meta just in two SQL queries
			*/
			$post_meta_infos = $lsdb->get_results( "SELECT meta_key, meta_value FROM $lsdb->postmeta WHERE post_id=$post_id" );
			if ( count( $post_meta_infos ) !== 0 ) {
				$sql_query = "INSERT INTO $lsdb->postmeta (post_id, meta_key, meta_value) ";
				foreach ( $post_meta_infos as $meta_info ) {
					$meta_key = $meta_info->meta_key;
					if ( '_wp_old_slug' !== $meta_key ) {
						continue;
					}
					$meta_value      = addslashes( $meta_info->meta_value );
					$sql_query_sel[] = "SELECT $new_post_id, '$meta_key', '$meta_value'";
				}
				$sql_query .= implode( ' UNION ALL ', $sql_query_sel );
				$lsdb->query( $sql_query );
			}

			/*
			* finally, redirect to the edit post screen for the new draft
			*/
			wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_post_id ) );
			exit();
		} else {
			wp_die( 'Post creation failed, could not find original post: ' . esc_attr( $post_id ) );
		}
	}

	/**
	* Save the variation metas
	*
	* @param type $variation_id Variation id.
	*/
	public function lsd_save_variation_settings_fields( $variation_id ) {
		$meta_key = 'lsd-metas';
		if ( isset( $_POST[ $meta_key ] ) && wp_verify_nonce( $_POST[ 'lsd_save_variation_config_nonce' ], basename( __FILE__ ) ) ) {
			$variation = wc_get_product( $variation_id );
			$old_metas = get_post_meta( $variation->parent->id, $meta_key, true );
			if ( empty( $old_metas ) ) {
				$old_metas = array();
			}
			$new_metas = array_replace( $old_metas, sanitize_text_field( wp_unslash( $_POST[ $meta_key ] ) ) );// Input var okay.
			update_post_meta( $variation->parent->id, $meta_key, $new_metas );
		}
	}

	/**
	* Get the config selector for each product
	*/
	public function lsd_get_product_config_selector() {
		$id = get_the_ID();

		$args        = array(
			'post_type' => 'lsd-config',
			'nopaging'  => true,
		);
		$configs     = get_posts( $args );
		$configs_ids = array( '' => 'None' );
		foreach ( $configs as $config ) {
			$configs_ids[ $config->ID ] = $config->post_title;
		}
		?>
		<div id="vpc_config_data" class="show_if_simple">
			<?php
			$this->lsd_get_product_tab_row( $id, $configs_ids, 'Configuration' );
			?>
		</div>
		<?php
	}

	/**
	* Add a new tab for the product to choose a config
	*
	* @param type $pid Product id.
	* @param type $configs_ids Configuration ids.
	* @param type $title Title.
	*/
	private function lsd_get_product_tab_row( $pid, $configs_ids, $title ) {
		$begin = array(
			'type' => 'sectionbegin',
			'id'   => 'lsd-metas-data',
		);

		$configurations = array(
			'title'   => $title,
			'name'    => "lsd-metas[$pid][config-id]",
			'type'    => 'select',
			'options' => $configs_ids,
		);

		$end      = array( 'type' => 'sectionend' );
		$settings = apply_filters(
			'vpc_product_tab_settings', array(
				$begin,
				$configurations,
				$end,
			)
		);
		//Escaping output.
		$allowed_tags = lsd_allowed_tags();
		$raw_output = o_admin_fields( $settings );
		wp_nonce_field( 'edit', 'lsd_save_product_config_nonce' );
		echo "<div class='vpc-product-config-row'>" . wp_kses( $raw_output, $allowed_tags) . '</div>';
	}

	/**
	* Set Variables product configuration form*
	*
	* @param type $loop Loop.
	* @param type $variation_data Variation data.
	* @param type $variation Variation object.
	*/
	public function lsd_get_variation_product_config_selector( $loop, $variation_data, $variation ) {
		$id = $variation->ID;

		$args        = array(
			'post_type' => 'lsd-config',
		);
		$configs     = get_posts( $args );
		$configs_ids = array( '' => 'None' );
		foreach ( $configs as $config ) {
			$configs_ids[ $config->ID ] = $config->post_title;
		}
		?>
		<tr>
			<td>
				<?php
				$this->lsd_get_product_tab_row( $id, $configs_ids, 'Configuration' );
				?>
			</td>
		</tr>
		<?php
	}

	/**
	*  Force the layout in the config to be one column
	*
	* @param array $columns Array of columns number.
	* @return int
	*/
	public function lsd_get_layout_columns( $columns ) {
		$columns['lsd-config'] = 1;
		return $columns;
	}

}
