<?php
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of class-lsd-product
 *
 */
class LSD_Product {

	public $variation_id;
	public $root_product_id;
	public $product;
	public $settings;
	public $variation_settings;

	public function __construct( $id ) {
		if ( $id ) {
			$this->root_product_id = $this->get_parent( $id );
			//If it's a variable product
			if ( $id !== $this->root_product_id ) {
				$this->variation_id = $id;
			} //Simple product and others
			else {
				$this->variation_id = $this->root_product_id;
			}

			$this->product = wc_get_product( $id );
			$config        = get_post_meta( $this->root_product_id, 'lsd-metas', true );

			if ( isset( $config[ $this->variation_id ] ) ) {
				$config_id = $config[ $this->variation_id ]['config-id'];
				if ( $config_id ) {
					$this->settings = get_post_meta( $config_id, 'lsd-metas', true );
					$product_metas  = get_post_meta( $this->root_product_id, 'lsd-metas', true );
					if ( isset( $product_metas['related-products'] ) ) {
						$this->settings['related-products'] = $product_metas['related-products'];
					}
					if ( isset( $product_metas['related-quantities'] ) ) {
						$this->settings['related-quantities'] = $product_metas['related-quantities'];
					}
				}
			}
		}
	}

	/**
	 * Returns a variation root product ID
	 * @param type $variation_id Variation ID
	 * @return int
	 */
	public function get_parent( $variation_id ) {
		$variable_product = wc_get_product( $variation_id );
		if ( ! $variable_product ) {
			return false;
		}
		if ( 'variation' !== $variable_product->get_type() ) {
			$product_id = $variation_id;
		} else {
			//$product_id=$variable_product->parent->id;
			$product_id = $variable_product->get_parent_id();
		}

		return $product_id;
	}

	/**
	 * Adds the Custom column to the default products list to help identify which ones are custom
	 * @param array $defaults Default columns
	 * @return array
	 */
	public function lsd_get_product_columns( $defaults ) {
		$defaults['is_customizable'] = __( 'Custom', 'lsd' );
		return $defaults;
	}

	/**
	 * Sets the Custom column value on the products list to help identify which ones are custom
	 * @param type $column_name Column name
	 * @param type $id Product ID
	 */
	public function lsd_get_products_columns_values( $column_name, $id ) {
		if ( 'is_customizable' === $column_name ) {
			$lsd_metas = get_post_meta( $id, 'lsd-metas', true );
			if ( isset( $lsd_metas[ $id ]['config-id'] ) && empty( $lsd_metas[ $id ]['config-id'] ) ) {
				$product = wc_get_product( $id );
				if ( $product->get_type() !== 'variable' ) {
					esc_attr_e( 'No', 'lsd' );
				} else {
					$configurable = false;
					foreach ( $lsd_metas as $id => $metas ) {
						if ( '' !== $metas['config-id'] && false === $configurable ) {
							$configurable = true;
						}
					}

					if ( $configurable ) {
						esc_attr_e( 'Yes', 'lsd' );
					} else {
						esc_attr_e( 'No', 'lsd' );
					}
				}
			} else {
				if ( ! isset( $lsd_metas[ $id ] ) ) {
					esc_attr_e( 'No', 'lsd' );
				} else {
					esc_attr_e( 'Yes', 'lsd' );
				}
			}
		}
	}

	/**
	 * verifies if the product is customizable
	 */
	public function is_customizable() {
		return ( ! empty( $this->settings ) );
	}

	/**
	 * Returns the customization page URL
	 * @global Array $lsd_settings
	 * @param int $design_index Saved design index to load
	 * @param mixed $cart_item_key Cart item key to edit
	 * @param int $order_item_id Order item ID to load
	 * @param int $tpl_id ID of the template to load
	 * @return String
	 */
	public function lsd_get_design_url( $design_index = false, $cart_item_key = false, $order_item_id = false, $tpl_id = false ) {
		global $lsd_settings;
		if ( $this->variation_id ) {
			$item_id = $this->variation_id;
		} else {
			$item_id = $this->root_product_id;
		}

		$options     = $lsd_settings['lsd-general-options'];
		$lsd_page_id = $options['lsd_page_id'];
		if ( function_exists( 'icl_object_id' ) ) {
			$lsd_page_id = icl_object_id( $lsd_page_id, 'page', false, ICL_LANGUAGE_CODE );
		}
		$lsd_page_url = '';
		if ( $lsd_page_id ) {
			$lsd_page_url = get_permalink( $lsd_page_id );
			if ( $item_id ) {
				$query = wp_parse_url( $lsd_page_url, PHP_URL_QUERY );
				// Returns a string if the URL has parameters or NULL if not
				if ( get_option( 'permalink_structure' ) ) {
					if ( substr( $lsd_page_url, -1 ) !== '/' ) {
						$lsd_page_url .= '/';
					}
					if ( $design_index || 0 === $design_index ) {
						$lsd_page_url .= "saved-design/$item_id/$design_index/";
					} elseif ( $cart_item_key ) {
						$lsd_page_url .= "edit/$item_id/$cart_item_key/";
					} elseif ( $order_item_id ) {
						$lsd_page_url .= "ordered-design/$item_id/$order_item_id/";
					} else {
						$lsd_page_url .= 'design/' . $item_id . '/';
						if ( $tpl_id ) {
							$lsd_page_url .= "$tpl_id/";
						}
					}
				} else {
					if ( false !== $design_index ) {
						$lsd_page_url .= '&product_id=' . $item_id . '&design_index=' . $design_index;
					} elseif ( $cart_item_key ) {
						$lsd_page_url .= '&product_id=' . $item_id . '&edit=' . $cart_item_key;
					} elseif ( $order_item_id ) {
						$lsd_page_url .= '&product_id=' . $item_id . '&oid=' . $order_item_id;
					} else {
						$lsd_page_url .= '&product_id=' . $item_id;
						if ( $tpl_id ) {
							$lsd_page_url .= "&tpl=$tpl_id";
						}
					}
				}
			}
		}

		return $lsd_page_url;
	}

	/**
	 * Returns a variation root product ID
	 * @param type $variation_id Variation ID
	 * @return int
	 */
	public function lsd_get_parent( $variation_id ) {
		$variable_product = wc_get_product( $variation_id );
		if ( ! $variable_product ) {
			return false;
		}
		if ( $variable_product->get_type() !== 'variation' ) {
			$product_id = $variation_id;
		} else {
			$product_id = $variable_product->get_parent_id();
		}

		return $product_id;
	}

	/**
	 * Save product settings fields
	 */
	public function lsd_save_product_settings_fields( $item_id ) {
		$meta_key = 'lsd-metas';
		if ( isset( $_POST[ $meta_key ] ) && wp_verify_nonce( $_POST[ 'lsd_save_product_config_nonce' ], 'edit' ) ) {
			$variation = wc_get_product( $item_id );
			//If we're dealing with a variation, Product ID is the root ID of the product
			if ( get_class( $variation ) === 'WC_Product_Variation' ) {
				$product_id = $variation->get_parent_id();
			} else {
				$product_id = $item_id;
			}
			//Careful this hooks only send the updated data, not the complete form
			$old_metas = get_post_meta( $product_id, $meta_key, true );
			if ( empty( $old_metas ) ) {
				$old_metas = array();
			}
			$new_metas = array_replace( $old_metas, $_POST[ $meta_key ] );
			update_post_meta( $product_id, $meta_key, $new_metas );
		}
	}

	/**
	 * Hide the cart button
	 */
	public function lsd_hide_cart_button() {
		global $product;
		global $lsd_settings;
		$general_options     = $lsd_settings['lsd-general-options'];
		$hide_cart_button    = get_proper_value( $general_options, 'lsd-hide-cart-button', true );
		$custom_products     = lsd_get_custom_products();
		$custom_products_ids = array_map( 'lsd_retrieve_custom_products_id', $custom_products );

		$pid = $product->get_id();
		if ( in_array( $pid, $custom_products_ids, true ) && $hide_cart_button ) {
			?>
			<script type="text/javascript">
				jQuery('[value="<?php echo esc_attr( $pid ); ?>"]').parent().find('.add_to_cart_button').hide();
				jQuery('[value="<?php echo esc_attr( $pid ); ?>"]').parent().find('.single_add_to_cart_button').hide();
			</script>
			<?php
		}
	}

	/**
	 * Hide the cart button
	 */
	public function lsd_get_custom_products_body_class( $classes, $class ) {
		if ( is_singular( array( 'product' ) ) ) {
			global $lsd_settings;
			$general_options     = $lsd_settings['lsd-general-options'];
			$hide_cart_button    = get_proper_value( $general_options, 'lsd-hide-cart-button', true );
			$custom_products     = lsd_get_custom_products();
			$custom_products_ids = array_map( 'lsd_retrieve_custom_products_id', $custom_products );
			$pid                 = get_the_ID();

			if ( in_array( $pid, $custom_products_ids, true ) ) {
				array_push( $classes, 'lsd-is-customizable' );
				if ( $hide_cart_button ) {
					array_push( $classes, 'lsd-hide-cart-button' );
				}
			}
		}
		return $classes;
	}

	/**
	 * Displays the product buttons
	 */
	public function lsd_get_buttons() {
		ob_start();
		$product = $this->product;

		$lsd_metas = $this->settings;

		if ( $this->variation_id ) {
			$item_id = $this->variation_id;
		} else {
			$item_id = $this->root_product_id;
		}

		if ( $product->get_type() === 'variable' ) {
			$variations = $product->get_available_variations();
			foreach ( $variations as $variation ) {
				if ( ! $variation['is_purchasable'] || ! $variation['is_in_stock'] ) {
					continue;
				}
				$lsd_product = new LSD_Product( $variation['variation_id'] );
                                $allowed_tags = lsd_allowed_tags();
                                $raw_output = $lsd_product->lsd_get_buttons();
                                echo wp_kses( $raw_output, $allowed_tags);
			}
		} else {
			?>
						<div class="lsd-buttons-wrap-<?php echo esc_attr( $product->get_type() ); ?>" data-id="<?php echo esc_attr( $this->variation_id ); ?>">
			<?php
			//Design from blank
			if ( isset( $lsd_metas ) && ! empty( $lsd_metas ) ) {
				$design_from_blank_url = $this->lsd_get_design_url();
				echo '<a  href="' . esc_url( $design_from_blank_url ) . '" class="mg-top-10 lsd-customize-product">' . esc_attr( 'Design', 'lsd' ) . '</a>';
			}
			?>
			</div>
				<?php
		}

			$output = ob_get_clean();
			return $output;
	}

	/**
	 * Displays the variation notice for config selection
	 */
	public function lsd_get_variable_product_details_location_notice() {
			?>
		<div class="options_group show_if_simple show_if_variable">
			<p class="form-field _sold_individually_field show_if_simple show_if_variable" style="background-color: red; color: white;">
		<?php esc_html_e( 'The variable product custom data for the product designer are shown by variation. In order to setup the product properties, please access the variations parameters tab.', 'lsd' ); ?>
			</p>
		</div>
		<?php
	}

	/**
	 * Duplicate the product metas while duplicating the product
	 */
	public function lsd_duplicate_product_metas( $new_product, $old_product ) {
		$meta_key  = 'lsd-metas';
		$old_metas = get_post_meta( $old_product->get_id(), $meta_key, true );

		$new_metas = lsd_replace_key_in_array( $old_metas, $old_product->get_id(), $new_product->get_id() );

		update_post_meta( $new_product->get_id(), $meta_key, $new_metas );
	}

	/**
	 * Save the config
	 */
	public function lsd_save_config( $post_id ) {
		$meta_key = 'lsd-metas';
		if ( isset( $_POST[ $meta_key ] ) && wp_verify_nonce( $_POST[ 'lsd_save_product_config_nonce' ], 'edit' ) ) {
			update_post_meta( $post_id, $meta_key, $_POST[ $meta_key ] );
		}
	}

}
